@@ -8,6 +8,65 @@ import functools
 import inspect
 import hashlib
 from typing import Optional, Any, Callable
+import logging
+
+
+def _serialize_for_key(obj):
+    """
+    Robust serialization function for cache key generation.
+    
+    This function ensures deterministic serialization of objects for cache keys:
+    - Handles JSON-serializable objects with sorted keys
+    - Falls back to string representation for complex objects
+    - Provides consistent output for identical inputs
+    
+    Args:
+        obj: Object to serialize
+        
+    Returns:
+        str: Serialized representation suitable for cache key generation
+    """
+    try:
+        return json.dumps(obj, sort_keys=True, default=str, ensure_ascii=True)
+    except (TypeError, ValueError) as e:
+        # Log warning for objects that might not be deterministic
+        if hasattr(obj, '__dict__') and not (hasattr(obj, '__repr__') or hasattr(obj, '__str__')):
+            logging.warning(f"Object {type(obj).__name__} may not generate consistent cache keys. "
+                          f"Consider implementing __repr__ or __str__ method.")
+        return str(obj)
+
+
+def _generate_cache_key(func_name: str, args: tuple, kwargs: dict) -> str:
+    """
+    Generate a deterministic cache key from function name and arguments.
+    
+    This function creates consistent hash keys for identical function calls:
+    - Function name ensures separation between different functions
+    - Arguments are serialized deterministically
+    - Keyword arguments are sorted to ensure order independence
+    - SHA-256 hash provides fixed-length, collision-resistant keys
+    
+    Args:
+        func_name: Name of the function being cached
+        args: Positional arguments tuple
+        kwargs: Keyword arguments dictionary
+        
+    Returns:
+        str: 64-character hexadecimal hash string
+    """
+    try:
+        key_parts = [
+            func_name,
+            _serialize_for_key(args),
+            _serialize_for_key(sorted(kwargs.items()) if kwargs else [])
+        ]
+        key_data = ''.join(key_parts).encode('utf-8')
+        return hashlib.sha256(key_data).hexdigest()
+    except Exception as e:
+        # Fallback to a basic key generation if serialization fails
+        logging.warning(f"Cache key generation failed for {func_name}: {e}. Using fallback method.")
+        fallback_key = f"{func_name}_{hash(str(args))}_{hash(str(sorted(kwargs.items()) if kwargs else []))}"
+        return hashlib.sha256(fallback_key.encode('utf-8')).hexdigest()
 
 
 class Condition:
@@ -222,27 +281,14 @@ def scache(ttl: Optional[int] = None, store: Optional[KVStore] = None, condition
         
         @functools.wraps(func)
         def wrapper(*args, **kwargs):
-            if cache_store.get(conditional_key, default=True) == False:
+            if conditional_key and cache_store.get(conditional_key, default=True) == False:
                 return func(*args, **kwargs)
             
-            # Generate cache key
+            # Generate cache key using robust function
             if key_func:
                 key = key_func(func, args, kwargs)
             else:
-                # Create cache key from function name and arguments
-                def serialize_for_key(obj):
-                    """Serialize object for cache key generation."""
-                    try:
-                        return json.dumps(obj, sort_keys=True, default=str)
-                    except (TypeError, ValueError):
-                        return str(obj)
-                
-                key_parts = [
-                    func.__name__,
-                    serialize_for_key(args),
-                    serialize_for_key(sorted(kwargs.items()))
-                ]
-                key = hashlib.sha256(''.join(key_parts).encode()).hexdigest()
+                key = _generate_cache_key(func.__name__, args, kwargs)
             
             # Try to get from cache
             cached_value = cache_store.get(key)
@@ -255,27 +301,14 @@ def scache(ttl: Optional[int] = None, store: Optional[KVStore] = None, condition
             return result
             
         async def async_wrapper(*args, **kwargs):
-            if cache_store.get(conditional_key, default=True) == False:
-                return func(*args, **kwargs)
+            if conditional_key and cache_store.get(conditional_key, default=True) == False:
+                return await func(*args, **kwargs)
             
-            # Generate cache key
+            # Generate cache key using robust function
             if key_func:
                 key = key_func(func, args, kwargs)
             else:
-                # Create cache key from function name and arguments
-                def serialize_for_key(obj):
-                    """Serialize object for cache key generation."""
-                    try:
-                        return json.dumps(obj, sort_keys=True, default=str)
-                    except (TypeError, ValueError):
-                        return str(obj)
-                
-                key_parts = [
-                    func.__name__,
-                    serialize_for_key(args),
-                    serialize_for_key(sorted(kwargs.items()))
-                ]
-                key = hashlib.sha256(''.join(key_parts).encode()).hexdigest()
+                key = _generate_cache_key(func.__name__, args, kwargs)
             
             # Try to get from cache
             cached_value = cache_store.get(key)
